@@ -7,7 +7,8 @@
 #' @param seasonal.factor integers; Multiple frequency integers considered for \link{NNS.ARMA} model, i.e. \code{(seasonal.factor = c(12, 24, 36))}
 #' @param negative.values logical; \code{FALSE} (default) If the variable can be negative, set to
 #' \code{(negative.values = TRUE)}.
-#' @param obj.fn expression; \code{expression(sum((predicted - actual)^2))} (default) Sum of squared errors is the default objective function.  Any \code{expression()} using the specific terms \code{predicted} and \code{actual} can be used.
+#' @param obj.fn expression;
+#' \code{expression(cor(predicted, actual, method = "spearman") / sum((predicted - actual)^2))} (default) Rank correlation / sum of squared errors is the default objective function.  Any \code{expression()} using the specific terms \code{predicted} and \code{actual} can be used.
 #' @param objective options: ("min", "max") \code{"min"} (default) Select whether to minimize or maximize the objective function \code{obj.fn}.
 #' @param linear.approximation logical; \code{TRUE} (default) Uses the best linear output from \code{NNS.reg} to generate a nonlinear and mixture regression for comparison.  \code{FALSE} is a more exhaustive search over the objective space.
 #' @param lin.only logical; \code{FALSE} (default) Restricts the optimization to linear methods only.
@@ -20,6 +21,7 @@
 #' \item{\code{$weights}} the optimal weights of each seasonal period between an equal weight or NULL weighting
 #' \item{\code{$obj.fn}} the objective function value
 #' \item{\code{$method}} the method identifying which \link{NNS.ARMA} method was used.
+#' \item{\code{$shrink}} whether to use the \code{shrink} parameter in \link{NNS.ARMA}.
 #' \item{\code{$bias.shift}} a numerical result of the overall bias of the optimum objective function result.  To be added to the final result when using the \link{NNS.ARMA} with the derived parameters.
 #'}
 #' @note
@@ -55,14 +57,14 @@
 #' @export
 
 NNS.ARMA.optim <- function(variable, training.set,
-                        seasonal.factor,
-                        negative.values = FALSE,
-                        obj.fn = expression(sum((predicted - actual)^2)),
-                        objective = "min",
-                        linear.approximation = TRUE,
-                        lin.only = FALSE,
-                        print.trace = TRUE,
-                        ncores = NULL){
+                           seasonal.factor,
+                           negative.values = FALSE,
+                           obj.fn = expression(cor(predicted, actual, method = "spearman") / sum((predicted - actual)^2)),
+                           objective = "max",
+                           linear.approximation = TRUE,
+                           lin.only = FALSE,
+                           print.trace = TRUE,
+                           ncores = NULL){
 
   if(any(class(variable)==c("tbl", "data.table"))) variable <- as.vector(unlist(variable))
 
@@ -74,9 +76,9 @@ NNS.ARMA.optim <- function(variable, training.set,
   objective <- tolower(objective)
 
   if (is.null(ncores)) {
-      num_cores <- as.integer(parallel::detectCores()) - 1
+    num_cores <- as.integer(parallel::detectCores()) - 1
   } else {
-      num_cores <- ncores
+    num_cores <- ncores
   }
 
   if(is.null(training.set)){stop("Please use the length of the variable less the forecast period as the training.set value.")}
@@ -112,36 +114,36 @@ NNS.ARMA.optim <- function(variable, training.set,
   if(lin.only) methods <- "lin" else methods <- c('lin','nonlin','both')
 
   for(j in methods){
-      current.seasonals <- list()
-      current.estimate <- numeric()
-      seasonal.combs <- list()
+    current.seasonals <- list()
+    current.estimate <- numeric()
+    seasonal.combs <- list()
 
-      for(i in 1 : length(seasonal.factor)){
-          nns.estimates.indiv <- list()
-
-          if(i == 1){
-              seasonal.combs[[i]] <- t(seasonal.factor)
-          } else {
-              remaining.index <- !(seasonal.factor%in%current.seasonals[[i-1]])
-              if(sum(remaining.index)==0){ break }
-              seasonal.combs[[i]] <- rbind(replicate(length(seasonal.factor[remaining.index]), current.seasonals[[i-1]]), as.integer(seasonal.factor[remaining.index]))
-          }
+    for(i in 1 : length(seasonal.factor)){
+      nns.estimates.indiv <- list()
 
       if(i == 1){
-          if(linear.approximation  && j!="lin"){
-              seasonal.combs[[1]] <- matrix(unlist(overall.seasonals[[1]]),ncol=1)
-              current.seasonals[[1]] <- unlist(overall.seasonals[[1]])
-
-          } else {
-              current.seasonals[[i]] <- as.integer(unlist(seasonal.combs[[1]]))
-          }
+        seasonal.combs[[i]] <- t(seasonal.factor)
       } else {
-          if(linear.approximation  && j!="lin"){
-              next
-          } else {
-              current.seasonals[[i]] <- as.integer(unlist(current.seasonals[[i-1]]))
+        remaining.index <- !(seasonal.factor%in%current.seasonals[[i-1]])
+        if(sum(remaining.index)==0){ break }
+        seasonal.combs[[i]] <- rbind(replicate(length(seasonal.factor[remaining.index]), current.seasonals[[i-1]]), as.integer(seasonal.factor[remaining.index]))
+      }
 
-          }
+      if(i == 1){
+        if(linear.approximation  && j!="lin"){
+          seasonal.combs[[1]] <- matrix(unlist(overall.seasonals[[1]]),ncol=1)
+          current.seasonals[[1]] <- unlist(overall.seasonals[[1]])
+
+        } else {
+          current.seasonals[[i]] <- as.integer(unlist(seasonal.combs[[1]]))
+        }
+      } else {
+        if(linear.approximation  && j!="lin"){
+          next
+        } else {
+          current.seasonals[[i]] <- as.integer(unlist(current.seasonals[[i-1]]))
+
+        }
       }
 
 
@@ -152,21 +154,21 @@ NNS.ARMA.optim <- function(variable, training.set,
 
       if(j!="lin" && linear.approximation){
 
-          # Find the min (obj.fn) for a given seasonals sequence
-          actual <- tail(variable, h)
+        # Find the min (obj.fn) for a given seasonals sequence
+        actual <- tail(variable, h)
 
-          predicted <- NNS.ARMA(variable, training.set = training.set, h = h, seasonal.factor = unlist(overall.seasonals[[1]]), method = j, plot = FALSE, negative.values = negative.values, ncores = 1)
+        predicted <- NNS.ARMA(variable, training.set = training.set, h = h, seasonal.factor = unlist(overall.seasonals[[1]]), method = j, plot = FALSE, negative.values = negative.values, ncores = 1)
 
-          nns.estimates.indiv <- eval(obj.fn)
+        nns.estimates.indiv <- eval(obj.fn)
 
       } else {
 
-          if(num_cores>1){
-            cl <- parallel::makeCluster(num_cores)
-            doParallel::registerDoParallel(cl)
-          }
+        if(num_cores>1){
+          cl <- parallel::makeCluster(num_cores)
+          doParallel::registerDoParallel(cl)
+        }
 
-          nns.estimates.indiv <- foreach(k = 1 : ncol(seasonal.combs[[i]]),.packages = c("NNS", "data.table"))%dopar%{
+        nns.estimates.indiv <- foreach(k = 1 : ncol(seasonal.combs[[i]]),.packages = c("NNS", "data.table"))%dopar%{
           actual <- tail(variable, h)
 
           predicted <- NNS.ARMA(variable, training.set = training.set, h = h, seasonal.factor =  seasonal.combs[[i]][ , k], method = j, plot = FALSE, negative.values = negative.values, ncores = 1)
@@ -176,8 +178,8 @@ NNS.ARMA.optim <- function(variable, training.set,
         }
 
         if(num_cores>1){
-            stopCluster(cl)
-            registerDoSEQ()
+          stopCluster(cl)
+          registerDoSEQ()
         }
 
       }
@@ -186,17 +188,17 @@ NNS.ARMA.optim <- function(variable, training.set,
       nns.estimates.indiv <- unlist(nns.estimates.indiv)
 
       if(objective=='min'){
-          nns.estimates.indiv[is.na(nns.estimates.indiv)] <- Inf
+        nns.estimates.indiv[is.na(nns.estimates.indiv)] <- Inf
       } else {
-          nns.estimates.indiv[is.na(nns.estimates.indiv)] <- -Inf
+        nns.estimates.indiv[is.na(nns.estimates.indiv)] <- -Inf
       }
 
       nns.estimates[[i]] <- nns.estimates.indiv
       nns.estimates.indiv <- numeric()
 
       if(objective=='min'){
-          current.seasonals[[i]] <- seasonal.combs[[i]][,which.min(nns.estimates[[i]])]
-          current.estimate[i] <- min(nns.estimates[[i]])
+        current.seasonals[[i]] <- seasonal.combs[[i]][,which.min(nns.estimates[[i]])]
+        current.estimate[i] <- min(nns.estimates[[i]])
 
         if(i > 1 && current.estimate[i] > current.estimate[i-1]){
           current.seasonals <- current.seasonals[-length(current.estimate)]
@@ -204,8 +206,8 @@ NNS.ARMA.optim <- function(variable, training.set,
           break
         }
       } else {
-          current.seasonals[[i]] <- seasonal.combs[[i]][,which.max(nns.estimates[[i]])]
-          current.estimate[i] <- max(nns.estimates[[i]])
+        current.seasonals[[i]] <- seasonal.combs[[i]][,which.max(nns.estimates[[i]])]
+        current.estimate[i] <- max(nns.estimates[[i]])
         if(i > 1 && current.estimate[i] < current.estimate[i-1]){
           current.seasonals <- current.seasonals[-length(current.estimate)]
           current.estimate <- current.estimate[-length(current.estimate)]
@@ -226,14 +228,14 @@ NNS.ARMA.optim <- function(variable, training.set,
 
       ### BREAKING PROCEDURE FOR IDENTICAL PERIODS ACROSS METHODS
       if(which(c("lin",'nonlin','both')==j) > 1 ){
-          if(sum(as.numeric(unlist(current.seasonals[[i]]))%in%as.numeric(unlist(previous.seasonals[[which(c("lin",'nonlin','both')==j)-1]][i])))==length(as.numeric(unlist(current.seasonals[[i]])))){
+        if(sum(as.numeric(unlist(current.seasonals[[i]]))%in%as.numeric(unlist(previous.seasonals[[which(c("lin",'nonlin','both')==j)-1]][i])))==length(as.numeric(unlist(current.seasonals[[i]])))){
 
-              if(objective=='min'){
-                  if(current.estimate[i] >= previous.estimates[[which(c("lin",'nonlin','both')==j)-1]][i]) break
-              } else {
-                  if(current.estimate[i] <= previous.estimates[[which(c("lin",'nonlin','both')==j)-1]][i]) break
-              }
+          if(objective=='min'){
+            if(current.estimate[i] >= previous.estimates[[which(c("lin",'nonlin','both')==j)-1]][i]) break
+          } else {
+            if(current.estimate[i] <= previous.estimates[[which(c("lin",'nonlin','both')==j)-1]][i]) break
           }
+        }
       }
 
       if(j!='lin' && linear.approximation){ break }
@@ -248,13 +250,13 @@ NNS.ARMA.optim <- function(variable, training.set,
 
 
     if(print.trace){
-        if(i > 1){
-            print(paste0("BEST method = ", paste0("'",j,"'"),  ", seasonal.factor = ", paste("c(", paste(unlist(current.seasonals[length(current.estimate)]), collapse = ", "))," )"))
-            print(paste0("BEST ", j, " OBJECTIVE FUNCTION = ", current.estimate[length(current.estimate)]))
-        } else {
-            print(paste0("BEST method = ", paste0("'",j,"'"), " PATH MEMBER = ", paste("c(", paste(unlist(current.seasonals), collapse = ", "))," )"))
-            print(paste0("BEST ", j, " OBJECTIVE FUNCTION = ", current.estimate[1]))
-        }
+      if(i > 1){
+        print(paste0("BEST method = ", paste0("'",j,"'"),  ", seasonal.factor = ", paste("c(", paste(unlist(current.seasonals[length(current.estimate)]), collapse = ", "))," )"))
+        print(paste0("BEST ", j, " OBJECTIVE FUNCTION = ", current.estimate[length(current.estimate)]))
+      } else {
+        print(paste0("BEST method = ", paste0("'",j,"'"), " PATH MEMBER = ", paste("c(", paste(unlist(current.seasonals), collapse = ", "))," )"))
+        print(paste0("BEST ", j, " OBJECTIVE FUNCTION = ", current.estimate[1]))
+      }
     }
 
     gc()
@@ -267,96 +269,106 @@ NNS.ARMA.optim <- function(variable, training.set,
 
 
   if(objective=='min'){
-      nns.periods <- unlist(overall.seasonals[[which.min(unlist(overall.estimates))]])
-      nns.method <- c("lin","nonlin","both")[which.min(unlist(overall.estimates))]
-      nns.SSE <- min(unlist(overall.estimates))
+    nns.periods <- unlist(overall.seasonals[[which.min(unlist(overall.estimates))]])
+    nns.method <- c("lin","nonlin","both")[which.min(unlist(overall.estimates))]
+    nns.SSE <- min(unlist(overall.estimates))
 
     if(length(nns.periods)>1){
-        predicted <- NNS.ARMA(variable, training.set = training.set, h = h, seasonal.factor = nns.periods, method = nns.method, plot = FALSE, negative.values = negative.values, ncores = 1, weights = rep((1/length(nns.periods)),length(nns.periods)))
+      predicted <- NNS.ARMA(variable, training.set = training.set, h = h, seasonal.factor = nns.periods, method = nns.method, plot = FALSE, negative.values = negative.values, ncores = 1, weights = rep((1/length(nns.periods)),length(nns.periods)))
 
-        weight.SSE <- eval(obj.fn)
+      weight.SSE <- eval(obj.fn)
 
-        if(weight.SSE<nns.SSE){
-            nns.weights <- rep((1/length(nns.periods)),length(nns.periods))
+      if(weight.SSE<nns.SSE){
+        nns.weights <- rep((1/length(nns.periods)),length(nns.periods))
 
-            bias <- mode(predicted - actual)
-            predicted <- predicted+bias
-            bias.SSE <- eval(obj.fn)
+        bias <- gravity(predicted - actual)
+        predicted <- predicted-bias
+        bias.SSE <- eval(obj.fn)
 
-            if(bias.SSE>weight.SSE){bias <- 0}
+        if(bias.SSE>weight.SSE){bias <- 0}
 
-        } else {
-            nns.weights <- NULL
-
-            bias <- mode(predicted - actual)
-            predicted <- predicted+bias
-            bias.SSE <- eval(obj.fn)
-
-            if(bias.SSE>nns.SSE){bias <- 0}
-        }
-    } else {
+      } else {
         nns.weights <- NULL
 
-        bias <- mode(predicted - actual)
-        predicted <- predicted+bias
+        bias <- gravity(predicted - actual)
+        predicted <- predicted-bias
         bias.SSE <- eval(obj.fn)
 
         if(bias.SSE>nns.SSE){bias <- 0}
+      }
+    } else {
+      nns.weights <- NULL
+
+      bias <- gravity(predicted - actual)
+      predicted <- predicted-bias
+      bias.SSE <- eval(obj.fn)
+
+      if(bias.SSE>nns.SSE){bias <- 0}
     }
 
   } else {
-      nns.periods <- unlist(overall.seasonals[[which.max(unlist(overall.estimates))]])
-      nns.method <- c("lin","nonlin","both")[which.max(unlist(overall.estimates))]
-      nns.SSE <- max(unlist(overall.estimates))
+    nns.periods <- unlist(overall.seasonals[[which.max(unlist(overall.estimates))]])
+    nns.method <- c("lin","nonlin","both")[which.max(unlist(overall.estimates))]
+    nns.SSE <- max(unlist(overall.estimates))
 
-      if(length(nns.periods)>1){
-          predicted <- NNS.ARMA(variable, training.set = training.set, h = h, seasonal.factor = nns.periods, method = nns.method, plot = FALSE, negative.values = negative.values, ncores = 1, weights = rep((1/length(nns.periods)),length(nns.periods)))
+    if(length(nns.periods)>1){
+      predicted <- NNS.ARMA(variable, training.set = training.set, h = h, seasonal.factor = nns.periods, method = nns.method, plot = FALSE, negative.values = negative.values, ncores = 1, weights = rep((1/length(nns.periods)),length(nns.periods)))
 
-          weight.SSE <- eval(obj.fn)
+      weight.SSE <- eval(obj.fn)
 
-          if(weight.SSE>nns.SSE){
-              nns.weights <- rep((1/length(nns.periods)),length(nns.periods))
+      if(weight.SSE>nns.SSE){
+        nns.weights <- rep((1/length(nns.periods)),length(nns.periods))
 
-              bias <- mode(predicted - actual)
-              predicted <- predicted+bias
-              bias.SSE <- eval(obj.fn)
+        bias <- gravity(predicted - actual)
+        predicted <- predicted-bias
+        bias.SSE <- eval(obj.fn)
 
-              if(bias.SSE<weight.SSE){bias <- 0}
+        if(bias.SSE<weight.SSE){bias <- 0}
 
-          } else {
-              nns.weights <- NULL
-
-              bias <- mode(predicted - actual)
-              predicted <- predicted+bias
-              bias.SSE <- eval(obj.fn)
-              if(objective=="min"){
-                if(bias.SSE<=nns.SSE){bias <- 0}
-              } else {
-                if(bias.SSE>=nns.SSE){bias <- 0}
-              }
-          }
       } else {
-          nns.weights <- NULL
-
-          bias <- mode(predicted - actual)
-          predicted <- predicted+bias
-          bias.SSE <- eval(obj.fn)
-          if(objective=="min"){
-              if(bias.SSE<=nns.SSE){bias <- 0}
-          } else {
-              if(bias.SSE>=nns.SSE){bias <- 0}
-          }
-
+        nns.weights <- NULL
 
+        bias <- gravity(predicted - actual)
+        predicted <- predicted-bias
+        bias.SSE <- eval(obj.fn)
+        if(objective=="min"){
+          if(bias.SSE<=nns.SSE){bias <- 0}
+        } else {
+          if(bias.SSE>=nns.SSE){bias <- 0}
+        }
       }
+    } else {
+      nns.weights <- NULL
+
+      bias <- gravity(predicted - actual)
+      predicted <- predicted-bias
+      bias.SSE <- eval(obj.fn)
+      if(objective=="min"){
+        if(bias.SSE<=nns.SSE){bias <- 0}
+      } else {
+        if(bias.SSE>=nns.SSE){bias <- 0}
+      }
+
+
+    }
 
   }
 
+  predicted <- NNS.ARMA(variable, training.set = training.set, h = h, seasonal.factor = nns.periods, method = nns.method, plot = FALSE, negative.values = negative.values, ncores = 1, weights = nns.weights, shrink = TRUE)
+
+  if(objective == "min") if(eval(obj.fn) < nns.SSE) nns.shrink = TRUE else nns.shrink = FALSE
+
+  if(objective == "max") if(eval(obj.fn) > nns.SSE) nns.shrink = TRUE else nns.shrink = FALSE
+
   options(warn = oldw)
 
+  if(!negative.values) bias <- min(c(bias, variable))
+
+
   return(list(periods = nns.periods,
               weights = nns.weights,
               obj.fn = nns.SSE,
               method = nns.method,
-              bias.shift = bias))
+              shrink = nns.shrink,
+              bias.shift = -bias))
 }
