@@ -177,15 +177,13 @@ NNS.boost <- function(IVs.train,
 
     for(i in 1:learner.trials){
       set.seed(123 + i)
-      new.index <- sample(length(y), as.integer(CV.size*length(y)), replace = FALSE)
 
-      if(i > 1){
-        maxes <- as.vector(apply(x, 2, which.max))
-        mins <- as.vector(apply(x, 2, which.min))
-        new.index_half <- new.index.1[1:(length(new.index.1)/2)]
-        new.index <- na.omit(unique(c(mins, maxes, new.index_half, new.index))[1:as.integer(CV.size*length(y))])
+      l <- length(y)
+      if(i<=l/4) new.index <- as.integer(seq(i, length(y), length.out = as.integer(CV.size * length(y)))) else {
+          new.index <- sample(l, as.integer(CV.size * l), replace = FALSE)
       }
 
+
       if(!is.null(ts.test)) new.index <- 1:(length(y) - ts.test)
 
       new.index <- unlist(new.index)
@@ -231,8 +229,6 @@ NNS.boost <- function(IVs.train,
         predicted <- pmax(predicted, min(as.numeric(y)))
       }
 
-      new.index.1 <- rev(order(abs(predicted - actual)))
-
       results[i] <- eval(obj.fn)
 
     } # i in learner.trials
@@ -290,15 +286,11 @@ NNS.boost <- function(IVs.train,
   if(!is.null(epochs)){
     for(j in 1:epochs){
       set.seed(123 * j)
-      new.index <- sample(length(y), as.integer(CV.size*length(y)), replace = FALSE)
 
-      if(j > 1){
-        maxes <- as.vector(apply(x, 2, which.max))
-        mins <- as.vector(apply(x, 2, which.min))
-        new.index_half <- new.index.1[1:(length(new.index.1)/2)]
-        new.index <- na.omit(unique(c(mins, maxes, new.index_half, new.index))[1:as.integer(CV.size*length(y))])
+      l <- length(y)
+      if(j<=l/4) new.index <- as.integer(seq(j, length(y), length.out = as.integer(CV.size * length(y)))) else {
+        new.index <- sample(l, as.integer(CV.size * l), replace = FALSE)
       }
-
       if(!is.null(ts.test)) new.index <- length(y) - (2*ts.test):0
 
       new.index <- unlist(new.index)
@@ -344,7 +336,6 @@ NNS.boost <- function(IVs.train,
         predicted <- pmax(predicted,min(as.numeric(y)))
       }
 
-      new.index.1 <- rev(order(abs(predicted - actual)))
 
       new.results <- eval(obj.fn)
 
